(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('apollo-link'), require('graphql/language/printer')) :
	typeof define === 'function' && define.amd ? define(['exports', 'apollo-link', 'graphql/language/printer'], factory) :
	(factory((global.persistedQueryLink = {}),global.apolloLink,global.graphql_language_printer));
}(this, (function (exports,apolloLink,graphql_language_printer) { 'use strict';

var __assign = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __rest = (undefined && undefined.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
var sha256 = require('hash.js/lib/hash/sha/256');
var VERSION = 1;
var defaultGenerateHash = function (query) {
    return sha256()
        .update(graphql_language_printer.print(query))
        .digest('hex');
};
var defaultOptions = {
    generateHash: defaultGenerateHash,
    disable: function (_a, _b) {
        var errors = _a.errors;
        var response = _b.response;
        if (errors.some(function (_a) {
            var message = _a.message;
            return message === 'PersistedQueryNotSupported';
        })) {
            return true;
        }
        if (response && response.statusCode && response.statusCode >= 500) {
            return true;
        }
        return false;
    },
};
var createPersistedQueryLink = function (_a) {
    var _b = _a === void 0 ? defaultOptions : _a, generateHash = _b.generateHash, disable = _b.disable;
    var supportsPersistedQueries = true;
    var calculated = new Map();
    return new apolloLink.ApolloLink(function (operation, forward) {
        var query = operation.query;
        var hashError;
        if (supportsPersistedQueries) {
            var hash = calculated.get(query);
            if (!hash) {
                try {
                    hash = generateHash(query);
                    calculated.set(query, hash);
                }
                catch (e) {
                    hashError = e;
                }
            }
            operation.extensions.persistedQuery = {
                version: VERSION,
                sha256Hash: hash,
            };
        }
        return new apolloLink.Observable(function (observer) {
            if (hashError) {
                observer.error(hashError);
                return;
            }
            var subscription;
            var tried = false;
            var handler = {
                next: function (_a) {
                    var data = _a.data, errors = _a.errors, rest = __rest(_a, ["data", "errors"]);
                    if (!tried && errors) {
                        supportsPersistedQueries = !disable(__assign({ data: data, errors: errors }, rest), operation.getContext());
                        tried = true;
                        if (subscription)
                            subscription.unsubscribe();
                        operation.setContext({
                            http: {
                                includeQuery: true,
                                includeExtensions: supportsPersistedQueries,
                            },
                        });
                        subscription = forward(operation).subscribe(handler);
                        return;
                    }
                    observer.next(__assign({ data: data, errors: errors }, rest));
                },
                error: observer.error.bind(observer),
                complete: observer.complete.bind(observer),
            };
            operation.setContext({
                http: {
                    includeQuery: !supportsPersistedQueries,
                    includeExtensions: supportsPersistedQueries,
                },
            });
            subscription = forward(operation).subscribe(handler);
            return function () {
                if (subscription)
                    subscription.unsubscribe();
            };
        });
    });
};

exports.VERSION = VERSION;
exports.defaultGenerateHash = defaultGenerateHash;
exports.defaultOptions = defaultOptions;
exports.createPersistedQueryLink = createPersistedQueryLink;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=bundle.umd.js.map
